@@ -11,9 +11,20 @@ except ImportError:  # pragma: no cover
         yield enter_result
 
 import six
+from django.contrib.admin.options import ModelAdmin
 from django.db.models.query import QuerySet
 from django.db.models.sql.query import Query
 
+try:  # pragma: no cover
+    from django.contrib.admin import validation as admin_validation  # noqa: F401
+except ImportError:  # pragma: no cover
+    admin_validation = None  # noqa: F401
+
+try:  # pragma: no cover
+    from django.core import checks  # noqa: F401
+except ImportError:  # pragma: no cover
+    checks = None  # noqa: F401
+
 try:  # pragma: no cover
     from django.db.models.constants import LOOKUP_SEP  # noqa: F401
 except ImportError:  # pragma: no cover
@@ -26,6 +37,11 @@ except ImportError:  # pragma: no cover
     from django.db.models.query import ValuesQuerySet  # noqa: F401
     ModelIterable = None
 
+try:  # pragma: no cover
+    from django.forms.utils import pretty_name  # noqa: F401
+except ImportError:  # pragma: no cover
+    from django.forms.forms import pretty_name  # noqa: F401
+
 # A dictionary mapping names of build_filter/add_filter keyword arguments to
 # keyword arguments for an _add_q/add_q call. It contains kwargs names for
 # all Django versions (some do not use all of these). If a keyword argument
@@ -80,6 +96,10 @@ if not hasattr(Query, 'annotation_select'):  # pragma: no cover
 QUERYSET_CHAIN_METHOD_NAME = '_chain' if hasattr(QuerySet, '_chain') else '_clone'
 QUERY_CHAIN_METHOD_NAME = 'chain' if hasattr(Query, 'chain') else 'clone'
 
+# The `get_queryset` method of ModelAdmins was called `queryset` in very old
+# Django versions.
+ADMIN_QUERYSET_METHOD_NAME = 'get_queryset' if hasattr(ModelAdmin, 'get_queryset') else 'queryset'
+
 
 def convert_build_filter_to_add_q_kwargs(**build_filter_kwargs):
     """
