@@ -8,8 +8,10 @@ from collections import namedtuple
 from copy import deepcopy
 from functools import wraps
 
+import six
 from django.core.exceptions import ObjectDoesNotExist
 from django.db.models import Q
+from django.utils.decorators import method_decorator
 from django.utils.tree import Node
 
 from . import get_queryable_property, MISSING_OBJECT
@@ -17,6 +19,60 @@ from ..compat import get_related_model, LOOKUP_SEP
 from ..exceptions import FieldDoesNotExist, QueryablePropertyDoesNotExist, QueryablePropertyError
 
 
+@six.python_2_unicode_compatible
+class QueryPath(tuple):
+    """
+    A utility class to represent query paths, i.e. paths using Django's
+    LOOKUP_SEP as their separator.
+
+    Objects can be used to build the string representation of a query path and
+    to combine paths using the `+` operator.
+    """
+
+    def __new__(cls, path=()):
+        """
+        Build a new query path instance using the given path, which may be
+        either a string that will be split up using the LOOKUP_SEP or another
+        type of iterable that already contains the individual path parts.
+
+        :param collections.Iterable path: The query path to represent as string
+                                          or other iterable.
+        """
+        if isinstance(path, six.string_types):
+            path = path.split(LOOKUP_SEP)
+        return super(QueryPath, cls).__new__(cls, path)
+
+    def __add__(self, other):
+        if not isinstance(other, self.__class__):
+            other = self.__class__(other)
+        return self.__class__(tuple(self) + tuple(other))
+
+    def __getitem__(self, item):
+        result = super(QueryPath, self).__getitem__(item)
+        if isinstance(item, slice):
+            result = self.__class__(result)
+        return result
+
+    def __getslice__(self, i, j):  # pragma: no cover
+        return self.__class__(super(QueryPath, self).__getslice__(i, j))
+
+    def __str__(self):
+        return LOOKUP_SEP.join(self)
+
+    def __repr__(self):
+        return '<{}: {}>'.format(self.__class__.__name__, six.text_type(self))
+
+    def build_filter(self, value):
+        """
+        Build a filter condition based on this query path and the given value.
+
+        :param value: The value to filter against.
+        :return: The filter condition as a Q object.
+        :rtype: django.db.models.Q
+        """
+        return Q(**{six.text_type(self): value})
+
+
 class QueryablePropertyReference(namedtuple('QueryablePropertyReference', 'property model relation_path')):
     """
     A reference to a queryable property that also holds the path to reach the
@@ -27,15 +83,13 @@ class QueryablePropertyReference(namedtuple('QueryablePropertyReference', 'prope
     @property
     def full_path(self):
         """
-        Return the full path to the queryable property (including the relation
-        prefix) in the query filter format.
+        Return the full query path to the queryable property (including the
+        relation prefix).
 
         :return: The full path to the queryable property.
-        :rtype: str
+        :rtype: QueryPath
         """
-        if not self.relation_path:
-            return self.property.name
-        return LOOKUP_SEP.join(self.relation_path + (self.property.name,))
+        return self.relation_path + self.property.name
 
     def get_filter(self, lookups, value):
         """
@@ -43,8 +97,7 @@ class QueryablePropertyReference(namedtuple('QueryablePropertyReference', 'prope
         points to. It checks if the property actually supports filtering and
         applies the relation path (if any) to the returned Q object.
 
-        :param collections.Sequence[str] lookups: The lookups/transforms to use
-                                                  for the filter.
+        :param QueryPath lookups: The lookups/transforms to use for the filter.
         :param value: The value passed to the filter condition.
         :return: A Q object to filter using this property.
         :rtype: django.db.models.Q
@@ -56,13 +109,13 @@ class QueryablePropertyReference(namedtuple('QueryablePropertyReference', 'prope
         # Use the model stored on this reference instead of the one on the
         # property since the query may be happening from a subclass of the
         # model the property is defined on.
-        q_obj = self.property.get_filter(self.model, LOOKUP_SEP.join(lookups) or 'exact', value)
+        q_obj = self.property.get_filter(self.model, six.text_type(lookups) or 'exact', value)
         if self.relation_path:
             # If the resolved property belongs to a related model, all actual
             # conditions in the returned Q object must be modified to use the
             # current relation path as prefix.
             def prefix_condition(item):
-                return LOOKUP_SEP.join(self.relation_path + (item[0],)), item[1]
+                return six.text_type(self.relation_path + item[0]), item[1]
             q_obj = TreeNodeProcessor(q_obj).modify_leaves(prefix_condition)
         return q_obj
 
@@ -112,20 +165,30 @@ class InjectableMixin(object):
     def mix_with_class(cls, base_class, class_name=None):
         """
         Create a new class based on the given base class and this mixin class.
-        The created class will also receive a custom :meth:`__reduce__`
-        implementation to make its objects picklable.
+        If this mixin class is already part of the class hierarchy of the given
+        base class, the base class will be returned unchanged.
 
         :param type base_class: The base class to mix the mixin into.
         :param str class_name: An optional name for the dynamically created
                                class. If None is supplied (default), the class
                                name of the dynamically created class will be
-                               the one of the object's original class.
-        :return: The generated class.
+                               the one of the object's original class. Will
+                               be applied if a new class is created.
+        :return: The generated class or the base class if it already uses this
+                 mixin.
         :rtype: type
         """
+        if issubclass(base_class, cls):
+            return base_class
+
         class_name = str(class_name or base_class.__name__)
         cache_key = (base_class, cls, class_name)
         created_class = cls._created_classes.get(cache_key)
+        metaclass = type
+        if not issubclass(cls.__class__, base_class.__class__) and not issubclass(base_class.__class__, cls.__class__):
+            # If the metaclasses of both classes are unrelated, try to build
+            # a new metaclass based on both dynamically.
+            metaclass = type(base_class.__class__.__name__, (cls.__class__, base_class.__class__), {})
         if created_class is None:
             attrs = {}
             if cls._dynamic_pickling:
@@ -135,7 +198,7 @@ class InjectableMixin(object):
                     return _unpickle_injected_object, (base_class, cls, class_name), get_state()
                 attrs['__reduce__'] = __reduce__
 
-            created_class = cls._created_classes[cache_key] = type(class_name, (cls, base_class), attrs)
+            created_class = cls._created_classes[cache_key] = metaclass(class_name, (cls, base_class), attrs)
         return created_class
 
     @classmethod
@@ -143,16 +206,23 @@ class InjectableMixin(object):
         """
         Update the given object's class by dynamically generating a new class
         based on the object's original class and this mixin class and changing
-        the given object into an object of this new class.
+        the given object into an object of this new class. If this mixin is
+        already part of the object's class hierarchy, its class will not
+        change.
 
         :param obj: The object whose class should be changed.
         :param str class_name: An optional name for the dynamically created
                                class. If None is supplied (default), the class
                                name of the dynamically created class will be
-                               the one of the object's original class.
+                               the one of the object's original class. Will
+                               be applied if a new class is created.
+        :return: The (potentially) modified object.
         """
-        obj.__class__ = cls.mix_with_class(obj.__class__, class_name)
-        obj.init_injected_attrs()
+        new_class = cls.mix_with_class(obj.__class__, class_name)
+        if new_class is not obj.__class__:
+            obj.__class__ = new_class
+            obj.init_injected_attrs()
+        return obj
 
 
 # This must be a standalone function for Python 2, where it could not be
@@ -174,8 +244,7 @@ def _unpickle_injected_object(base_class, mixin_class, class_name=None):
              object's state).
     """
     obj = base_class.__new__(base_class, ())
-    mixin_class.inject_into_object(obj, class_name)
-    return obj
+    return mixin_class.inject_into_object(obj, class_name)
 
 
 _unpickle_injected_object.__safe_for_unpickling__ = True
@@ -261,7 +330,7 @@ class ModelAttributeGetter(object):
                                    will simply be replaced with the lookup
                                    separator (``__``).
         """
-        self.path_parts = attribute_path.split(self.ATTRIBUTE_SEPARATOR)
+        self.query_path = QueryPath(attribute_path.split(self.ATTRIBUTE_SEPARATOR))
 
     def get_value(self, obj):
         """
@@ -283,7 +352,7 @@ class ModelAttributeGetter(object):
                                            value for.
         :return: The value retrieved from the model object.
         """
-        for attribute_name in self.path_parts:
+        for attribute_name in self.query_path:
             try:
                 obj = getattr(obj, attribute_name)
             except ObjectDoesNotExist:
@@ -308,51 +377,52 @@ class ModelAttributeGetter(object):
         :return: The filter condition as a Q object.
         :rtype: django.db.models.Q
         """
-        return Q(**{LOOKUP_SEP.join(self.path_parts + [lookup]): value})
+        return (self.query_path + lookup).build_filter(value)
 
 
-def parametrizable_decorator(method):
+def parametrizable_decorator(function):
     """
-    A decorator for methods (not regular functions!) who themselves are to be
-    used as decorators and are to support both a parameter-less decorator usage
-    (``@my_decorator``) as well as parametrized decorator usage
-    (``@my_decorator(some_param=5)``). This decorator takes care of making the
-    distinction between both usages and returning the correct object.
+    A decorator for functions who themselves are to be used as decorators and
+    are to support both a parameter-less decorator usage (``@my_decorator``) as
+    well as parametrized decorator usage (``@my_decorator(some_param=5)``).
+    This decorator takes care of making the distinction between both usages and
+    returning the correct object.
 
-    :param function method: The decorator method to decorate.
-    :return: A wrapper method that will replace the decorated method.
+    :param function function: The decorator function to decorate.
+    :return: A wrapper function that will replace the decorated function.
     :rtype: function
     """
-    @wraps(method)
-    def wrapper(self, function=None, *args, **kwargs):
+    @wraps(function)
+    def wrapper(decorated_function=None, *args, **kwargs):
         def decorator(func):
-            return method(self, func, *args, **kwargs)
+            return function(func, *args, **kwargs)
 
-        if function:  # A function was given directly -> apply the decorator directly (@my_decorator usage).
-            return decorator(function)
+        if decorated_function:  # A function was given directly -> apply the decorator directly (@my_decorator usage).
+            return decorator(decorated_function)
         return decorator  # No function -> return the actual decorator (@my_decorator(some_param=5) usage).
     return wrapper
 
 
-def resolve_queryable_property(model, path):
+parametrizable_decorator_method = method_decorator(parametrizable_decorator)
+
+
+def resolve_queryable_property(model, query_path):
     """
     Resolve the given path into a queryable property on the given model.
 
     :param type model: The model to start resolving from.
-    :param collections.Sequence[str] path: The path to resolve (a string of
-                                           Django's query expression split
-                                           up by the lookup separator).
+    :param QueryPath query_path: The query path to resolve.
     :return: A 2-tuple containing a queryable property reference for the
-             resolved property and a list containing the parts of the path
-             that represent lookups (or transforms). The first item will be
-             None and the list will be empty if no queryable property could
-             be resolved.
-    :rtype: (QueryablePropertyReference, list[str])
+             resolved property and a query path containing the parts of the
+             path that represent lookups (or transforms). The first item will
+             be None and the query path will be empty if no queryable property
+             could be resolved.
+    :rtype: (QueryablePropertyReference, QueryPath)
     """
-    property_ref, lookups = None, []
+    property_ref, lookups = None, QueryPath()
     # Try to follow the given path to allow to use queryable properties
     # across relations.
-    for index, name in enumerate(path):
+    for index, name in enumerate(query_path):
         try:
             related_model = get_related_model(model, name)
         except FieldDoesNotExist:
@@ -363,8 +433,8 @@ def resolve_queryable_property(model, path):
                 # invalid name. Do nothing and let Django deal with it.
                 pass
             else:
-                property_ref = QueryablePropertyReference(prop, model, tuple(path[:index]))
-                lookups = path[index + 1:]
+                property_ref = QueryablePropertyReference(prop, model, query_path[:index])
+                lookups = query_path[index + 1:]
             # The current name was not a field and either a queryable
             # property or invalid. Either way, resolving ends here.
             break
